# -*- coding: utf-8 -*-
# passerelle - uniform access to multiple data sources and services
# Copyright (C) 2018 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import functools
import json
from urllib.parse import urlparse, parse_qs

import httmock
import pytest

import utils

from passerelle.contrib.rsa13.models import RSA13Resource


@pytest.fixture
def rsa13(db):
    return utils.make_resource(
        RSA13Resource,
        title='Test',
        slug='test',
        description='Test',
        webservice_base_url='https://rsa-cd13.com',
        basic_auth_username='username',
        basic_auth_password='password',
    )


@pytest.fixture
def url(rsa13):
    return '/rsa13/%s/' % rsa13.slug


def mock_response(*path_contents):
    def decorator(func):
        @httmock.urlmatch()
        def error(url, request):
            assert False, 'request to %s' % (url,)

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            handlers = []
            for row in path_contents:
                try:
                    path, query, content = row
                except ValueError:
                    path, content = row
                    query = None

                @httmock.urlmatch(path=path)
                def handler(url, request):
                    if query:
                        assert parse_qs(url.query) == parse_qs(query), 'query does not match'
                    return json.dumps(content)

                handlers.append(handler)
            handlers.append(error)

            with httmock.HTTMock(*handlers):
                return func(*args, **kwargs)

        return wrapper

    return decorator


@mock_response(['/api/check', {'ping': 'pong'}])
def test_check_status(rsa13):
    rsa13.check_status()


EMAIL = 'john.doe@example.com'
IP = '127.0.0.1'


@pytest.fixture
def app(app):
    '''Add authentication params email/ip to all requests'''
    orig_do_request = app.do_request

    def do_request(req, *args, **kwargs):
        qs = 'email=%s&ip=%s' % (EMAIL, IP)
        if req.environ['QUERY_STRING']:
            req.environ['QUERY_STRING'] += '&' + qs
        else:
            req.environ['QUERY_STRING'] = qs
        return orig_do_request(req, *args, **kwargs)

    app.do_request = do_request
    return app


MOTICLODAC_CODES = [
    {'id': 'ABSSUIVI', 'text': 'Aucune info. sur le suivi'},
    {'id': 'CADUQUE', 'text': 'Caduque'},
    {'id': 'REALISEE', 'text': 'Réalisée'},
    {'id': 'RUPTINJUST', 'text': 'Rupture injustifiée'},
    {'id': 'RUPTJUSTIF', 'text': 'Rupture justifiée'},
]


@mock_response(['/api/cg_ref_code/domain/MOTICLODAC', {'err': 0, 'data': MOTICLODAC_CODES}])
def test_nomenclature(app, rsa13, url):
    response = app.get(url + 'nomenclature/MOTICLODAC/')
    assert response.json == {
        'err': 0,
        'data': MOTICLODAC_CODES,
    }


PLATFORM_LIST = [{'dsp': 'LA', 'id': 11, 'name': 'ADPEI', 'role': 'Coordonnateur'}]


@mock_response(['/api/platform/', {'err': 0, 'data': PLATFORM_LIST}])
def test_platform(app, rsa13, url):
    response = app.get(url + 'platform/')
    assert response.json['data'] == PLATFORM_LIST


PLATFORM_DETAIL = {
    "id": 11,
    "name": "ADPEI",
    "dsp": "LA",
    "adr1": "ADPEI",
    "adr2": None,
    "adr3": None,
    "adr4": "18 BOULEVARD CAMILLE FLAMMARION",
    "adr5": None,
    "adr6": "13001 MARSEILLE",
    "tel": "0491110140",
    "queries": [
        {"id": 1, "name": "NON CONSULTÉ", "count": 727},
        {"id": 2, "name": "SANS RU", "count": 727},
        {"id": 3, "name": "SANS CONTRAT SUR LA PLATEFORME", "count": 231},
        {"id": 4, "name": "CONTRAT BIENTOT TERMINE", "count": 0},
        {"id": 5, "name": "SANS AUCUNE ACTION", "count": 44},
        {"id": 6, "name": "ACTION VALIDEE NON DEBUTEE NON CLOSE", "count": 111},
    ],
}


@mock_response(['/api/platform/11/', {'err': 0, 'data': PLATFORM_DETAIL}])
def test_platform_details(app, rsa13, url):
    response = app.get(url + 'platform/11/')
    assert response.json == {
        'err': 0,
        'data': PLATFORM_DETAIL,
    }


PLATFORM_REFERENT = [
    {
        "id": 324,
        "nom": "EHRMANN ",
        "prenom": "Jean Paul",
        "tel": None,
        "email": "john.doe@example.com",
        "role": "Coordonnateur",
        "statut": "Actif",
    },
    {
        "id": 239,
        "nom": "CHAUMONT ",
        "prenom": "Nadine",
        "tel": '090909090909',
        "email": "jane.doe@example.com",
        "role": "Accompagnateur",
        "statut": "Actif",
    },
]


@mock_response(['/api/platform/11/referent/', {'err': 0, 'data': PLATFORM_REFERENT}])
def test_platform_referent(app, rsa13, url):
    response = app.get(url + 'platform/11/referent/')
    assert response.json == {
        'err': 0,
        'data': PLATFORM_REFERENT,
    }


@pytest.mark.xfail
def test_platform_referent_post(app, rsa13, url):
    response = app.post_json(
        url + 'platform/11/referent/',
        params={
            'email': 'john.doe@example.com',
            'prenom': 'John',
            'nom': 'Doe',
            'tel': '0909090909',
        },
    )
    assert response.json == {}


BENEFICIAIRE = {
    "id": 386981,
    "civilite": "MR",
    "nom": "AAABEFBAADF",
    "prenom": "ACCDCBE",
    "date_naissance": "1958-01-01",
    "actif": "Oui",
    "matricule": "193740",
    "code_postal": "13001",
    "commune": "MARSEILLE",
    "code_pi": "51",
    "referent": " ",
    "date_deb_affectation": "2019-03-11",
    "consulte": "Oui",
    "toppersdrodevorsa": "Oui",
}


@mock_response(['/api/platform/11/beneficiaire/', 'nom=AAABEFBAADF', {'err': 0, 'data': [BENEFICIAIRE]}])
def test_platform_beneficiaire_nom(app, rsa13, url):
    response = app.get(url + 'platform/11/beneficiaire/', params={'nom': 'AAABEFBAADF'})
    assert response.json == {
        "err": 0,
        "data": [BENEFICIAIRE],
    }


@mock_response(['/api/platform/11/beneficiaire/', 'matricule=193740', {'err': 0, 'data': [BENEFICIAIRE]}])
def test_platform_beneficiaire_matricule(app, rsa13, url):
    response = app.get(url + 'platform/11/beneficiaire/', params={'matricule': '193740'})
    assert response.json == {
        "err": 0,
        "data": [BENEFICIAIRE],
    }


@mock_response(['/api/platform/11/beneficiaire/', 'page=0', {'err': 0, 'data': [BENEFICIAIRE]}])
def test_platform_beneficiaire_page(app, rsa13, url):
    response = app.get(url + 'platform/11/beneficiaire/', params={'page': '0'})
    assert response.json == {
        "err": 0,
        "data": [BENEFICIAIRE],
    }


@mock_response(
    [
        '/api/platform/11/beneficiaire/',
        'page=0&nom=NOM&prenom=PRENOM&matricule=MATRICULE&referent=REFERENT',
        {'err': 0, 'data': [BENEFICIAIRE]},
    ]
)
def test_platform_beneficiaire_full(app, rsa13, url):
    response = app.get(
        url + 'platform/11/beneficiaire/',
        params={
            'page': '0',
            'nom': 'NOM',
            'prenom': 'PRENOM',
            'matricule': 'MATRICULE',
            'referent': 'REFERENT',
        },
    )
    assert response.json == {
        "err": 0,
        "data": [BENEFICIAIRE],
    }


BENEFICIAIRE_DETAIL = {
    'actif': 'Oui',
    'adresse': {
        'adr2': '8 RUE MISSION DE FRANCE',
        'adr3': '13001 MARSEILLE',
        'adr4': None,
        'adr5': None,
        'adr6': None,
    },
    'age': '62 ans',
    'civilite': 'MR',
    'code_pi': '51',
    'commentaire_ref': None,
    'conjoint': {
        'age': '52 ans',
        'id': 854379,
        'nom': 'AAABEFBAADF',
        'plateforme': 'PI51',
        'prenom': 'EEFBEFEC',
    },
    'date_de_naissance': '1958-01-01',
    'droit': {
        'date_demande': '2008-10-16',
        'etat': 'Droit ouvert et versable',
        'motif': None,
        'toppersdrodevorsa': 'Oui',
    },
    'enfants': [
        {'age': '16 ans', 'nom': 'DBFFFCDDDDCBAEAC', 'prenom': 'DCACBEAFFD'},
        {'age': '14 ans', 'nom': 'DBFFFCDDDDCBAEAC', 'prenom': 'BEDAEDEEBBCD'},
    ],
    'id': 386981,
    'lib_code_pi': "Pôle d'Insertion Marseille I",
    'matricule': '193740',
    'nom': 'AAABEFBAADF',
    'nomnaiss': 'EAEFEFDDDBEFDF',
    'numdemrsa': '13378013',
    'prenom': 'ACCDCBE',
    'referent': {'id': None, 'nom': None, 'prenom': None},
    'situation_familiale': {'date_debut': '2007-10-01', 'libelle': 'VIE MARITALE'},
}


@mock_response(['/api/platform/11/beneficiaire/386981/', {'err': 0, 'data': BENEFICIAIRE_DETAIL}])
def test_platform_beneficiaire_detail(app, rsa13, url):
    response = app.get(url + 'platform/11/beneficiaire/386981/')
    assert response.json == {
        'err': 0,
        'data': BENEFICIAIRE_DETAIL,
    }


BENEFICIAIRE_TRANSPORT = {'cumuls': [{'duree': 54, 'type': 'GTU'}]}


@mock_response(
    ['/api/platform/11/beneficiaire/386981/transport/', {'err': 0, 'data': BENEFICIAIRE_TRANSPORT}]
)
def test_platform_beneficiaire_transport(app, rsa13, url):
    response = app.get(url + 'platform/11/beneficiaire/386981/transport/')
    assert response.json == {'err': 0, 'data': BENEFICIAIRE_TRANSPORT}


BENEFICIAIRE_CONTRAT = [
    {
        'clos': 'Non',
        'date_deb': '2019-09-18',
        'date_fin': '2020-03-18',
        'decision': 'Validé',
        'duree': 6,
        'id': 15,
        'operateur': 'ADPEI',
        'plateforme': 'ADPEI',
        'referent': 'EHRMANN  J',
        'retab': 'Non',
    },
    {
        'clos': 'Oui',
        'date_deb': '2019-03-21',
        'date_fin': '2019-09-21',
        'decision': 'Validé',
        'duree': 6,
        'id': 14,
        'operateur': 'ADPEI',
        'plateforme': 'ADPEI',
        'referent': 'EHRMANN  J',
        'retab': 'Non',
    },
    {
        'clos': 'Oui',
        'date_deb': '2009-04-03',
        'date_fin': '2009-10-02',
        'decision': 'Validé',
        'duree': 6,
        'id': 0,
        'operateur': 'REPRISE HISTORIQUE LOGIFORM',
        'plateforme': 'REPRISE HISTORIQUE LOGIFORM',
        'referent': ' ',
        'retab': 'Non',
    },
]


@mock_response(['/api/platform/11/beneficiaire/386981/contrat/', {'err': 0, 'data': BENEFICIAIRE_CONTRAT}])
def test_platform_beneficiaire_contrat(app, rsa13, url):
    response = app.get(url + 'platform/11/beneficiaire/386981/contrat/')
    assert response.json == {
        'err': 0,
        'data': BENEFICIAIRE_CONTRAT,
    }

BENEFICIAIRE_CONTRAT_DETAIL = {
    'commentaire': 'Gratuité RTM refusée. En tant que bénéficiaire de la '
    'CMUC vous pouvez obtenir un tarif préférentiel sur '
    'le réseau RTM.',
    'date_clos': None,
    'date_cvs': None,
    'date_deb': '2019-09-18',
    'date_fin': '2020-03-18',
    'date_retab': None,
    'date_sign': '2019-09-03',
    'decision': 'Validé',
    'duree': 6,
    'id': 15,
    'motif_cvs': None,
    'operateur': 'ADPEI',
    'plateforme': 'ADPEI',
    'referent': {'commentaire': None, 'nom': 'EHRMANN ', 'prenom': 'Jean Paul'},
    'type_contrat': "Contrat d'Engagement Réciproque-Social",
}


@mock_response(['/api/platform/11/beneficiaire/386981/contrat/15/', {'err': 0, 'data': BENEFICIAIRE_CONTRAT_DETAIL}])
def test_platform_beneficiaire_contrat_detail(app, rsa13, url):
    response = app.get(url + 'platform/11/beneficiaire/386981/contrat/15/')
    assert response.json == {
        'err': 0,
        'data': BENEFICIAIRE_CONTRAT_DETAIL,
    }
